@@ -1,7 +1,9 @@
 # Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 # SPDX-License-Identifier: Apache-2.0
+import copy
 import functools
 import sys
+import time
 from glob import glob
 from pathlib import Path
 
@@ -24,6 +26,8 @@ import os
 from threading import Thread
 
 import yaml
+
+import boto3
 from betterboto import client as betterboto_client
 from jinja2 import Environment, FileSystemLoader
 
@@ -116,43 +120,6 @@ def write_share_template(portfolio_use_by_account, region, host_account_id, shar
         )
 
 
-def create_share_template(deployment_map, import_map, puppet_account_id):
-    logger.info("deployment_map: {}".format(deployment_map))
-    ALL_REGIONS = get_regions()
-    for region in ALL_REGIONS:
-        logger.info("starting to build shares for region: {}".format(region))
-        with betterboto_client.ClientContextManager('servicecatalog', region_name=region) as servicecatalog:
-            portfolio_ids = {}
-
-            response = servicecatalog.list_portfolios_single_page()
-
-            for portfolio_detail in response.get('PortfolioDetails'):
-                portfolio_ids[portfolio_detail.get('DisplayName')] = portfolio_detail.get('Id')
-
-            logger.info("Portfolios in use in region: {}".format(portfolio_ids))
-
-            portfolio_use_by_account = {}
-            for account_id, launch_details in deployment_map.items():
-                if portfolio_use_by_account.get(account_id) is None:
-                    portfolio_use_by_account[account_id] = []
-                for launch_id, launch in launch_details.get('launches').items():
-                    p = portfolio_ids[launch.get('portfolio')]
-                    if p not in portfolio_use_by_account[account_id]:
-                        portfolio_use_by_account[account_id].append(p)
-
-            for account_id, import_details in import_map.items():
-                if portfolio_use_by_account.get(account_id) is None:
-                    portfolio_use_by_account[account_id] = []
-                for spoke_local_portfolio_id, spoke_local_portfolio in import_details.get('spoke-local-portfolios').items():
-                    p = portfolio_ids[spoke_local_portfolio.get('portfolio')]
-                    if p not in portfolio_use_by_account[account_id]:
-                        portfolio_use_by_account[account_id].append(p)
-
-            host_account_id = response.get('PortfolioDetails')[0].get('ARN').split(":")[4]
-            sharing_policies = generate_bucket_policies_for_shares(deployment_map, puppet_account_id)
-            write_share_template(portfolio_use_by_account, region, host_account_id, sharing_policies)
-
-
 template_dir = asset_helpers.resolve_from_site_packages('templates')
 env = Environment(
     loader=FileSystemLoader(template_dir),
@@ -160,6 +127,7 @@ env = Environment(
 )
 
 
+@functools.lru_cache(maxsize=32)
 def get_puppet_account_id():
     with betterboto_client.ClientContextManager('sts') as sts:
         return sts.get_caller_identity().get('Account')
@@ -176,7 +144,7 @@ def set_regions_for_deployment_map(deployment_map, section, puppet_account_id):
                 for a in launch_details.get('deploy_to').get('accounts'):
                     if a.get('account_id') == account_id:
                         regions = a.get('regions')
-                        if regions in ["enabled", "regions_enabled"]:
+                        if regions in ["enabled", "regions_enabled", "enabled_regions"]:
                             regions = account_details.get('regions_enabled')
                         elif regions == "default_region" or regions is None:
                             regions = account_details.get('default_region')
@@ -199,7 +167,7 @@ def set_regions_for_deployment_map(deployment_map, section, puppet_account_id):
                 for t in launch_details.get('deploy_to').get('tags'):
                     if t.get('tag') in account_details.get('tags'):
                         regions = t.get('regions')
-                        if regions in ["enabled", "regions_enabled"]:
+                        if regions in ["enabled", "regions_enabled", "enabled_regions"]:
                             regions = account_details.get('regions_enabled')
                         elif regions == "default_region" or regions is None:
                             regions = account_details.get('default_region')
@@ -292,9 +260,17 @@ def wire_dependencies(all_tasks):
         for dependency in task.get('depends_on', []):
             for task_uid_2, task_2 in all_tasks.items():
                 if task_2.get('launch_name') == dependency:
-                    task.get('dependencies').append(task_2)
+                    new_task = copy.deepcopy(task_2)
+                    if new_task.get('depends_on') is not None:
+                        del new_task['depends_on']
+                    task.get('dependencies').append(new_task)
         del task['depends_on']
         tasks_to_run.append(task)
+
+        for task_to_run in tasks_to_run:
+            for dependency in task_to_run.get('dependencies', []):
+                dependency['dependencies'] = []
+
     return tasks_to_run
 
 
@@ -574,13 +550,12 @@ def deploy_launches_task_builder(deployment_map, manifest, puppet_account_id, se
     return all_tasks
 
 
-@functools.lru_cache(512)
-def get_required_params(region_name, portfolio, product, version, puppet_account_id):
-    logging.info(f"get_required_params for {region_name} {portfolio} {product} {version}")
+def get_required_params(region_name, portfolio, product, version, account_id):
+    logging.info(f"get_required_params for {region_name} {portfolio} {product} {version} {account_id}")
     with betterboto_client.CrossAccountClientContextManager(
             'servicecatalog',
-            f"arn:aws:iam::{puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
-            f"{puppet_account_id}-{region_name}",
+            f"arn:aws:iam::{account_id}:role/servicecatalog-puppet/PuppetRole",
+            f"{account_id}-{region_name}",
             region_name=region_name
     ) as service_catalog:
         portfolio_id = aws.get_portfolio_id_for(service_catalog, portfolio)
@@ -606,15 +581,13 @@ def deploy_launches_task_builder_for_account_launch_region(
         puppet_account_id, region_name
 ):
     all_tasks = {}
-
     required_parameters = get_required_params(
         region_name,
         launch_details.get('portfolio'),
         launch_details.get('product'),
         launch_details.get('version'),
-        puppet_account_id,
+        account_id,
     )
-
     regular_parameters, ssm_parameters = get_parameters_for_launch(
             required_parameters,
             deployment_map,
@@ -653,12 +626,14 @@ def deploy_launches_task_builder_for_account_launch_region(
     if launch_details.get('configuration'):
         if launch_details.get('configuration').get('retry_count'):
             task['retry_count'] = launch_details.get('configuration').get('retry_count')
+        if launch_details.get('configuration').get('requested_priority'):
+            task['requested_priority'] = launch_details.get('configuration').get('requested_priority')
 
     all_tasks[f"{task.get('account_id')}-{task.get('region')}-{task.get('launch_name')}"] = task
     return all_tasks
 
 
-def run_tasks(tasks_to_run):
+def run_tasks(tasks_to_run, num_workers):
     should_use_eventbridge = get_should_use_eventbridge(os.environ.get("AWS_DEFAULT_REGION"))
     should_forward_failures_to_opscenter = get_should_forward_failures_to_opscenter(os.environ.get("AWS_DEFAULT_REGION"))
 
@@ -672,13 +647,16 @@ def run_tasks(tasks_to_run):
     for type in ["failure", "success", "timeout", "process_failure", "processing_time", "broken_task", ]:
         os.makedirs(Path(constants.RESULTS_DIRECTORY) / type)
 
+    logger.info(f"About to run workflow with {num_workers} workers")
+
     run_result = luigi.build(
         tasks_to_run,
         local_scheduler=True,
         detailed_summary=True,
-        workers=10,
+        workers=num_workers,
         log_level='INFO',
     )
+
     table_data = [
         ['Action','Launch', 'Account', 'Region', 'Portfolio', 'Product', 'Version', 'Duration'],
 
@@ -759,7 +737,11 @@ def run_tasks(tasks_to_run):
     if should_use_eventbridge:
         logging.info(f"Sending {len(entries)} events to eventbridge")
         with betterboto_client.ClientContextManager('events') as events:
-            events.put_events(Entries=entries)
+            for i in range(0, len(entries), constants.EVENTBRIDGE_MAX_EVENTS_PER_CALL):
+                events.put_events(
+                    Entries=entries[i:i+constants.EVENTBRIDGE_MAX_EVENTS_PER_CALL]
+                )
+                time.sleep(1)
         logging.info(f"Finished sending {len(entries)} events to eventbridge")
     sys.exit(exit_status_codes.get(run_result.status))
 
@@ -821,3 +803,105 @@ def run_tasks_for_dry_run(tasks_to_run):
         ])
     click.echo(table.table)
     sys.exit(exit_status_codes.get(run_result.status))
+
+
+def run_tasks_for_generate_shares(tasks_to_run):
+    for type in ["failure", "success", "timeout", "process_failure", "processing_time", "broken_task", ]:
+        os.makedirs(Path(constants.RESULTS_DIRECTORY) / type)
+
+    run_result = luigi.build(
+        tasks_to_run,
+        local_scheduler=True,
+        detailed_summary=True,
+        workers=10,
+        log_level='INFO',
+    )
+
+    should_use_sns = get_should_use_sns()
+    puppet_account_id = get_puppet_account_id()
+    version = get_puppet_version()
+
+    for region in get_regions():
+        sharing_policies = {
+            'accounts': [],
+            'organizations': [],
+        }
+        with betterboto_client.ClientContextManager('cloudformation', region_name=region) as cloudformation:
+            cloudformation.ensure_deleted(StackName="servicecatalog-puppet-shares")
+
+            logger.info(f"generating policies collection for region {region}")
+            if os.path.exists(os.path.sep.join(['data','bucket'])):
+                logger.info(f"Updating policies for the region: {region}")
+                path = os.path.sep.join(['data','bucket', region, 'accounts'])
+                if os.path.exists(path):
+                    for account_file in os.listdir(path):
+                        account = account_file.split(".")[0]
+                        sharing_policies['accounts'].append(account)
+
+                path = os.path.sep.join(['data','bucket', region, 'organizations'])
+                if os.path.exists(path):
+                    for organization_file in os.listdir(path):
+                        organization = organization_file.split(".")[0]
+                        sharing_policies['organizations'].append(organization)
+
+            logger.info(f"Finished generating policies collection")
+
+            template = env.get_template('policies.template.yaml.j2').render(
+                sharing_policies=sharing_policies,
+                VERSION=version,
+            )
+            with betterboto_client.ClientContextManager('cloudformation', region_name=region) as cloudformation:
+                cloudformation.create_or_update(
+                    StackName="servicecatalog-puppet-policies",
+                    TemplateBody=template,
+                    NotificationARNs=[
+                        f"arn:aws:sns:{region}:{puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events"
+                    ] if should_use_sns else [],
+                )
+
+    for filename in glob('results/failure/*.json'):
+        result = json.loads(open(filename, 'r').read())
+        click.echo(colorclass.Color("{red}" + result.get('task_type') + " failed{/red}"))
+        click.echo(f"{yaml.safe_dump({'parameters':result.get('task_params')})}")
+        click.echo("\n".join(result.get('exception_stack_trace')))
+        click.echo('')
+    exit_status_codes = {
+        LuigiStatusCode.SUCCESS: 0,
+        LuigiStatusCode.SUCCESS_WITH_RETRY: 0,
+        LuigiStatusCode.FAILED: 1,
+        LuigiStatusCode.FAILED_AND_SCHEDULING_FAILED: 2,
+        LuigiStatusCode.SCHEDULING_FAILED: 3,
+        LuigiStatusCode.NOT_RUN: 4,
+        LuigiStatusCode.MISSING_EXT: 5,
+    }
+    sys.exit(exit_status_codes.get(run_result.status))
+
+
+def run_tasks_for_bootstrap_spokes_in_ou(tasks_to_run):
+    for type in ["failure", "success", "timeout", "process_failure", "processing_time", "broken_task", ]:
+        os.makedirs(Path(constants.RESULTS_DIRECTORY) / type)
+
+    run_result = luigi.build(
+        tasks_to_run,
+        local_scheduler=True,
+        detailed_summary=True,
+        workers=10,
+        log_level='INFO',
+    )
+
+    for filename in glob('results/failure/*.json'):
+        result = json.loads(open(filename, 'r').read())
+        click.echo(colorclass.Color("{red}" + result.get('task_type') + " failed{/red}"))
+        click.echo(f"{yaml.safe_dump({'parameters':result.get('task_params')})}")
+        click.echo("\n".join(result.get('exception_stack_trace')))
+        click.echo('')
+    exit_status_codes = {
+        LuigiStatusCode.SUCCESS: 0,
+        LuigiStatusCode.SUCCESS_WITH_RETRY: 0,
+        LuigiStatusCode.FAILED: 1,
+        LuigiStatusCode.FAILED_AND_SCHEDULING_FAILED: 2,
+        LuigiStatusCode.SCHEDULING_FAILED: 3,
+        LuigiStatusCode.NOT_RUN: 4,
+        LuigiStatusCode.MISSING_EXT: 5,
+    }
+    sys.exit(exit_status_codes.get(run_result.status))
