@@ -47,11 +47,33 @@ def cli(info, info_line_numbers):
 
 def generate_shares(f):
     logger.info('Starting to generate shares for: {}'.format(f.name))
-
+    tasks_to_run = []
+    puppet_account_id = cli_command_helpers.get_puppet_account_id()
     manifest = manifest_utils.load(f)
+
     deployment_map = manifest_utils.build_deployment_map(manifest, constants.LAUNCHES)
+    for account_id, deployment_map_for_account in deployment_map.items():
+        tasks_to_run.append(
+            luigi_tasks_and_targets.CreateSharesForAccountImportMapTask(
+                account_id=account_id,
+                puppet_account_id=puppet_account_id,
+                deployment_map_for_account=deployment_map_for_account,
+                sharing_type='launches',
+            )
+        )
+
     import_map = manifest_utils.build_deployment_map(manifest, constants.SPOKE_LOCAL_PORTFOLIOS)
-    cli_command_helpers.create_share_template(deployment_map, import_map, cli_command_helpers.get_puppet_account_id())
+    for account_id, import_map_for_account in import_map.items():
+        tasks_to_run.append(
+            luigi_tasks_and_targets.CreateSharesForAccountImportMapTask(
+                account_id=account_id,
+                puppet_account_id=puppet_account_id,
+                deployment_map_for_account=import_map_for_account,
+                sharing_type='spoke-local-portfolios',
+            )
+        )
+
+    cli_command_helpers.run_tasks_for_generate_shares(tasks_to_run)
 
 
 def dry_run(f):
@@ -96,10 +118,10 @@ def reset_provisioned_product_owner(f):
         if task_status == constants.PROVISIONED:
             tasks_to_run.append(luigi_tasks_and_targets.ResetProvisionedProductOwnerTask(**task))
 
-    cli_command_helpers.run_tasks(tasks_to_run)
+    cli_command_helpers.run_tasks(tasks_to_run, 10)
 
 
-def deploy(f, single_account):
+def deploy(f, num_workers):
     puppet_account_id = cli_command_helpers.get_puppet_account_id()
 
     manifest = manifest_utils.load(f)
@@ -112,7 +134,6 @@ def deploy(f, single_account):
     all_launch_tasks = cli_command_helpers.deploy_launches(manifest, puppet_account_id)
     launch_tasks.update(all_launch_tasks)
 
-
     for task in cli_command_helpers.wire_dependencies(launch_tasks):
         task_status = task.get('status')
         del task['status']
@@ -142,7 +163,7 @@ def deploy(f, single_account):
     )
     tasks_to_run += spoke_local_portfolio_tasks_to_run
 
-    cli_command_helpers.run_tasks(tasks_to_run)
+    cli_command_helpers.run_tasks(tasks_to_run, num_workers)
 
 
 def bootstrap_spoke_as(puppet_account_id, iam_role_arns):
@@ -597,3 +618,49 @@ def remove_from_launches(launch_name):
     manifest = get_manifest()
     del manifest.get('launches')[launch_name]
     save_manifest(manifest)
+
+
+def set_config_value(name, value):
+    with betterboto_client.ClientContextManager('ssm', region_name=constants.HOME_REGION) as ssm:
+        try:
+            response = ssm.get_parameter(Name=constants.CONFIG_PARAM_NAME)
+            config = yaml.safe_load(response.get('Parameter').get('Value'))
+        except ssm.exceptions.ParameterNotFound:
+            config = {}
+
+        if name == "regions":
+            config['regions'] = value if len(value) > 1 else value[0].split(",")
+        else:
+            config[name] = value.upper() == 'TRUE'
+
+        upload_config(config)
+
+
+def bootstrap_spokes_in_ou(ou_path_or_id, role_name, iam_role_arns):
+    org_iam_role_arn = cli_command_helpers.get_org_iam_role_arn()
+    puppet_account_id = cli_command_helpers.get_puppet_account_id()
+    if org_iam_role_arn is None:
+        click.echo('No org role set - not expanding')
+    else:
+        click.echo('Expanding using role: {}'.format(org_iam_role_arn))
+        with betterboto_client.CrossAccountClientContextManager(
+                'organizations', org_iam_role_arn, 'org-iam-role'
+        ) as client:
+            tasks = []
+            if ou_path_or_id.startswith('/'):
+                ou_id = client.convert_path_to_ou(ou_path_or_id)
+            else:
+                ou_id = ou_path_or_id
+            logging.info(f"ou_id is {ou_id}")
+            response = client.list_children_nested(ParentId=ou_id, ChildType='ACCOUNT')
+            for spoke in response:
+                tasks.append(
+                    luigi_tasks_and_targets.BootstrapSpokeAsTask(
+                        puppet_account_id=puppet_account_id,
+                        account_id=spoke.get('Id'),
+                        iam_role_arns=iam_role_arns,
+                        role_name=role_name,
+                    )
+                )
+
+        cli_command_helpers.run_tasks_for_bootstrap_spokes_in_ou(tasks)
